import subprocess
import re
import os
import glob
from datetime import datetime, timedelta
from natsort import natsorted

def get_user_from_file(interface):
    """从配置文件中提取宽带账号"""
    file_path = f"/etc/sysconfig/network-scripts/ifcfg-{interface}"
    try:
        with open(file_path, 'r') as file:
            for line in file:
                if line.startswith("USER="):
                    return line.split('=')[1].strip()
    except FileNotFoundError:
        print(f"File {file_path} not found.")
    return "N/A"

def get_linkname_backup(interface):
    """通过IP地址查询linkname"""
    try:
        # 构建命令
        command = f"/usr/sbin/pppoe-status /etc/sysconfig/network-scripts/ifcfg-{interface} | awk '/global/ {{print $NF}}'"
        # 执行命令并获取输出
        result = subprocess.check_output(command, shell=True).decode('utf-8').strip()
        
        # 如果结果不为空，返回linkname
        if result:
            return result
        else:
            return "N/A"
    except subprocess.CalledProcessError as e:
        print(f"Error fetching linkname for {interface}: {e}")
        return "N/A"
    except ValueError as e:
        print(f"Error parsing linkname for {interface}: {e}")
        return "N/A"

def get_linkname(ip_address):
    """通过IP地址查询linkname"""
    try:
        # 构建命令
        command = f"ip addr | grep -E '{ip_address}($| )' | awk '{{print $7}}'"
        
        # 执行命令并获取输出
        result = subprocess.check_output(command, shell=True).decode('utf-8').strip()
        
        # 如果结果不为空，返回linkname
        if result:
            return result
        else:
            return "N/A"
    except subprocess.CalledProcessError as e:
        print(f"Error fetching linkname for {ip_address}: {e}")
        return "N/A"
    except ValueError as e:
        print(f"Error parsing linkname for {ip_address}: {e}")
        return "N/A"


def get_pid(user, interface):
    """通过ps命令查询PID号"""
    try:
        # 构建命令
        command = f"ps -aux | grep pppd | grep -E '{interface}($| )' | grep {user} | grep -v 'grep' | awk '{{print $2}}'"
        
        # 执行命令并获取输出
        result = subprocess.check_output(command, shell=True).decode('utf-8').strip()
        
        # 如果结果不为空，返回PID
        if result:
            return int(result)
        else:
            return "N/A"
    except subprocess.CalledProcessError as e:
        print(f"Error fetching PID for {interface}: {e}")
        return "N/A"
    except ValueError as e:
        print(f"Error parsing PID for {interface}: {e}")
        return "N/A"

def get_ip_address(interface):
    """通过接口名查询对应的IP地址"""
    try:
        # 构建命令
        command = f"/usr/sbin/pppoe-status /etc/sysconfig/network-scripts/ifcfg-{interface} | awk '$1 == \"inet\" {{print $2}}'"
        
        # 执行命令并获取输出
        result = subprocess.check_output(command, shell=True).decode('utf-8').strip()
        
        # 如果结果不为空，返回IP地址
        if result:
            return result
        else:
            return "N/A"
    except subprocess.CalledProcessError as e:
        print(f"Error fetching IP address for {interface}: {e}")
        return "N/A"

def get_ip_addressv6(interface):
    """通过接口名查询对应的IP地址"""
    try:
        # 构建命令
        command = f"/usr/sbin/pppoe-status /etc/sysconfig/network-scripts/ifcfg-{interface} | awk '$1 == \"inet6\" && $4 == \"global\" {{print $2}}'"
        
        # 执行命令并获取输出
        result = subprocess.check_output(command, shell=True).decode('utf-8').strip()
        
        # 如果结果不为空，返回IP地址
        if result:
            return result
        else:
            return "N/A"
    except subprocess.CalledProcessError as e:
        print(f"Error fetching IP address for {interface}: {e}")
        return "N/A"

def get_dial_status(pid, log_file):
    """通过PID号获取拨号反馈信息"""
    status = "N/A"
    try:
        # 使用 grep 命令过滤包含特定进程ID的行
        result = subprocess.check_output(['cat', log_file]).decode('utf-8')
        lines = result.split('\n')
        for line in lines:
            if f"{pid}" in line and "authentication" in line:
                # 使用正则表达式提取不同的 authentication 状态及其后面的消息作为状态
                succeeded_match = re.search(r'authentication\s*(.+)', line)
                pending_match = re.search(r'authentication\s+succeeded:\s*(.+)', line)
                failed_match = re.search(r'authentication\s+failed:\s*(.+)', line)
                timeout_match = re.search(r'authentication\s+timeout:\s*(.+)', line)

                if succeeded_match:
                    status = " " + succeeded_match.group(1)
                    break
                elif failed_match:
                    status = " " + failed_match.group(1)
                    break
                elif pending_match:
                    status = " " + pending_match.group(1)
                    break
                elif timeout_match:
                    status = " " + timeout_match.group(1)
                    break
    except subprocess.CalledProcessError as e:
        print(f"Error fetching dial status: {e}")
    return status

def get_interfaces():
    """获取接口编号"""
    interface_files = glob.glob("/etc/sysconfig/network-scripts/ifcfg-ppp*")
    interfaces = [os.path.basename(f).replace("ifcfg-", "") for f in interface_files]
    return interfaces

def get_log_file():
    # 获取当前日期并格式化为字符串
    current_date = datetime.now().strftime("%Y%m%d")
    log_file = f"/var/log/pppoe-{current_date}.log"

    # 如果当前日志文件不存在或为空，则使用前一天的日志文件
    if not os.path.exists(log_file) or os.path.getsize(log_file) == 0:
        previous_date = (datetime.now() - timedelta(days=1)).strftime("%Y%m%d")
        log_file = f"/var/log/pppoe-{previous_date}.log"
        
        # 如果前一天的日志文件也不存在或为空，则返回 None
        if not os.path.exists(log_file) or os.path.getsize(log_file) == 0:
            return None

    return log_file

def main():
    interfaces = get_interfaces()
    results = []

    log_file = get_log_file()  # 获取日志文件路径

    if log_file is None:
        print("找不到今天或昨天的有效日志文件")
        return
        
    for interface in interfaces:
        user = get_user_from_file(interface)
        pid = get_pid(user, interface)
        ip_address = get_ip_address(interface)
        ip_addressv6 = get_ip_addressv6(interface)
        dial_status = get_dial_status(pid, log_file)
        linkname = get_linkname(ip_address)

        results.append({
            "linkname": linkname,
            "interface": interface,
            "user": user,
            "pid": pid,
            "ip_address": ip_address,
            "ip_addressv6": ip_addressv6,
            "dial_status": dial_status
        })
    
    # 打印结果
    sorted_results = natsorted(results, key=lambda x: x['interface'])
    print(f"{'linkname':<10} {'ifcfg接口':<10} {'USER':<24} {'PID':<8} {'IP地址v4':<20} {'IP地址v6':<40} {'日志':<20}")
    for result in sorted_results:
        print(f"{result['linkname']:<10} {result['interface']:<10} {result['user']:<24} {result['pid']:<8} {result['ip_address']:<20} {result['ip_addressv6']:<40} {result['dial_status']:<20}")

if __name__ == "__main__":
    main()
